/**
 * Java SendMail
 * Copyright (C) 2005  Ondrej Vondra
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * Pouzita literatura (kdyby to nekoho zajimalo):
 * rfc  821 - SIMPLE MAIL TRANSFER PROTOCOL
 * rfc 1521 - MIME (Multipurpose Internet Mail Extensions) Part One:
 *                 Mechanisms for Specifying and Describing
 *                  the Format of Internet Message Bodies
 */

package sendmail;

import attachments.*;
import java.io.*;
import java.net.*;
import sugar.Sys;

public class cEntry {
  static final String banner = "Java SendMail by Ondrej Vondra";
  static final String usage =
      "<recipient> <SMTP server> <sender> [<files_to_attach>]";

  private static String[] attach;
  private static String recipient, sender, server, subject;

  private static boolean parse_arguments(String[] args)
  {
    if (args.length < 3)
      return false;
    recipient = args[0];
    server = args[1];
    sender = args[2];
    if (args.length > 3) {
      attach = new String[args.length - 3];
      System.arraycopy(args, 3, attach, 0, args.length - 3);
    }
    else {
      attach = null;
    }
    return true;
  }

  private static cAttachment[] createAttachments() throws Exception
  {
    if (attach == null)
      return null;

    FileNameMap fnmap = URLConnection.getFileNameMap();

    cAttachment r[] = new cAttachment[attach.length];

    for (int i=0; i<attach.length; i++)
    {
      File f = new File(attach[i]);
      if (!((f.exists()) && (f.isFile())))
        throw new Exception("Not existing or not regular file " + attach[i] + " is being attached.");
      FileInputStream fi = new FileInputStream(f);
      r[i] = new cAttachment(f.getName(), fi, (fnmap != null) ? fnmap.getContentTypeFor(attach[i]) : "application/unknown");
    }
    return r;
  }

  public static void main(String[] args)
  {
    try {
      System.out.println(banner);

      if (!parse_arguments(args))
      {
        throw new Exception("Usage: "+usage);
      }

      System.out.print("Subject: ");
      subject = Sys.readLine();

      cTerminalReader term = new cTerminalReader();

      cMailMessage msg = new cMailMessage(recipient, server, sender, subject);
      if (attach != null)
        msg.setAttachments(createAttachments());

      System.out.println("Start mail input terminate with <CR>.<CR>");
      msg.setBody(term);
      msg.send();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }
}