/**
 * Java SendMail
 * Copyright (C) 2005  Ondrej Vondra
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sendmail;

import attachments.*;
import java.io.*;
import java.net.*;
import java.util.*;

public class cMailMessage {

  private cSMTPConnection con;
  private InputStream body=null;
  private cAttachment[] attach=null;
  private String boundary;

  private void generateBoundary()
  {
    Random rnd = new Random(new Date().getTime());
    boundary = new String("=_NextPart_");
    for (int i=0; i<16; i++)
      boundary += Integer.toHexString(rnd.nextInt(127));
  }

  public cMailMessage(String recipient, String smtpserver, String sender, String subject) throws Exception
  {
    InetAddress locala = InetAddress.getLocalHost();
    String local = locala.getHostName();

    con = new cSMTPConnection(smtpserver, local);
    con.newmail(sender, recipient);
    con.sendmailline("From: "+sender);
    con.sendmailline("To: "+recipient);
    con.sendmailline("Subject: "+subject);
  }

  public void setBody(InputStream data)
  /*
   data must contain text/plain
   */
  {
    body = data;
  }

  public void setAttachments(cAttachment[] att) // resets list of attachments
  {
    attach = att;
  }

  public void send() throws Exception
  {
    String l;
    // we will encode body like it was a text/plain attachment
    cAttachment bodya = new cAttachment("", body, "text/plain");

    if (body == null)
      throw new Exception("No message body specified.");

    if ((attach!=null) && (attach.length>0)) // multipart message
    {
      generateBoundary();
      con.sendmailline("MIME-Version: 1.0");
      con.sendmailline("Content-Type: multipart/mixed; boundary=\""+
                       boundary+"\"");

      con.sendmailline("");
      con.sendmailline("This is a multi-part message in MIME format.");
      con.sendmailline("");

      con.sendmailline("--"+boundary);
      con.sendmailline("Content-Type: text/plain");
      con.sendmailline("Content-Transfer-Encoding: " + bodya.transferEncoding);
      con.sendmailline("Content-Disposition: inline");
      con.sendmailline("");
      while ((l=bodya.readLine())!=null)
        con.sendmailline(l);

      System.out.println("Encoding attachments ...");
      for (int ai=0; ai<attach.length; ai++)
      {
        System.out.println(""+(ai+1)+"/"+attach.length);
        con.sendmailline("");
        con.sendmailline("--"+boundary);
        con.sendmailline("Content-Type: "+attach[ai].contentType+"; name = \""+
                         attach[ai].name+"\"");
        con.sendmailline("Content-Transfer-Encoding: " + attach[ai].transferEncoding);
        con.sendmailline("Content-Disposition: attachment; filename=\""+
                         attach[ai].name+"\"");
        con.sendmailline("");

        while ((l = attach[ai].readLine())!=null)
        {
          con.sendmailline(l);
        }
      }
      System.out.println("Terminating message ...");
      con.sendmailline("");
      con.sendmailline("--"+boundary+"--");
      con.sendmailline("");
    }
    else // no attachments
    {
      con.sendmailline("Content-Type: text/plain");
      con.sendmailline("Content-Transfer-Encoding: " + bodya.transferEncoding);
      con.sendmailline("");
      while ((l=bodya.readLine())!=null)
        con.sendmailline(l);
    }
    con.endmail();
  }
}