/**
 * Java SendMail
 * Copyright (C) 2005  Ondrej Vondra
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sendmail;

import java.net.*;
import java.io.*;

public class cSMTPConnection {

  final String nl = "\r\n";

  private Socket socket;
  private Reader sockin;
  private Writer sockout;
  private String lastrecvline = new String("");
  private boolean mailbegun;

//  private BufferedWriter debugw;

  private String recvline() throws Exception
  {
    String s = new String("");

    while (true) {
      int ch = sockin.read();
      if (ch == '\r')
        continue;
      if ((ch =='\n') || (ch==-1))
        break;
      s += (char)ch;
    }

    // debug
    System.out.println("recvline: "+s);
    //

    lastrecvline = s;

    return s;
  }

  private boolean is_response_ok(String response)
  {
    return response.matches("250 .*");
  }

  private void sendline(String line) throws Exception
  {
//    debugw.write(line+nl);
    sockout.write(line+nl);
    sockout.flush();
  }

  public cSMTPConnection(String host, String localhost) throws Exception
  {
/*   File debugf = new File("debug.txt");
    if (debugf.exists())
      debugf.delete();
    debugf.createNewFile();
    debugw = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(debugf)));
*/
    mailbegun = false;

    socket = new Socket(host, 25);
    sockin = new InputStreamReader(socket.getInputStream());
    sockout = new OutputStreamWriter(socket.getOutputStream());

    recvline();
    sendline("HELO "+localhost);
    if (!is_response_ok(recvline()))
      throw new Exception("SMTP error response: "+lastrecvline);
  }

  public void newmail(String sender, String recipient) throws Exception
  {
    sendline("MAIL FROM:<" + sender + ">");
    if (!is_response_ok(recvline()))
      throw new Exception("SMTP error response: "+lastrecvline);

    sendline("RCPT TO:<"+recipient+">");
    if (!is_response_ok(recvline()))
      throw new Exception("SMTP error response: "+lastrecvline);

    sendline("DATA");
    recvline();
    mailbegun = true;
  }

  public void sendmailline(String line) throws Exception
  {
    if (!mailbegun)
      throw new Exception("No mail begun.");
    if (line.matches("\\..*"))
      line = "."+line;
    sendline(line);
  }

  private void hangup() throws Exception
  {
    sendline("QUIT");
    socket.shutdownOutput();
    socket.shutdownInput();
    socket.close();
  }

  public void endmail() throws Exception
  {
    if (mailbegun)
      sendline(nl+".");
    mailbegun = false;
    if (!is_response_ok(recvline()))
      throw new Exception("SMTP error response: "+lastrecvline);
    hangup();
  }

  protected void finalize() throws Exception
  {
    hangup();
  }
}